@@ -1,29 +1,37 @@
+import logging
 import os
 from contextlib import contextmanager
 from functools import wraps
 
-from dvc.ignore import CleanTree
-from dvc.compat import fspath_py35
-
 from funcy import cached_property, cat, first
 
 from dvc.config import Config
+from dvc.dvcfile import PIPELINE_FILE, Dvcfile, is_valid_filename
+from dvc.exceptions import FileMissingError
+from dvc.exceptions import IsADirectoryError as DvcIsADirectoryError
 from dvc.exceptions import (
-    FileMissingError,
-    IsADirectoryError,
+    NoOutputOrStageError,
     NotDvcRepoError,
     OutputNotFoundError,
 )
+from dvc.ignore import CleanTree
 from dvc.path_info import PathInfo
-from dvc.remote.base import RemoteActionNotImplemented
+from dvc.repo.tree import RepoTree
+from dvc.scm.tree import is_working_tree
 from dvc.utils.fs import path_isin
+
+from ..stage.exceptions import StageFileDoesNotExistError, StageNotFound
+from ..utils import parse_target
 from .graph import check_acyclic, get_pipeline, get_pipelines
 
+logger = logging.getLogger(__name__)
+
 
 def locked(f):
     @wraps(f)
     def wrapper(repo, *args, **kwargs):
         with repo.lock, repo.state:
+            # pylint: disable=protected-access
             repo._reset()
             ret = f(repo, *args, **kwargs)
             # Our graph cache is no longer valid after we release the repo.lock
@@ -33,7 +41,7 @@ def locked(f):
     return wrapper
 
 
-class Repo(object):
+class Repo:
     DVC_DIR = ".dvc"
 
     from dvc.repo.destroy import destroy
@@ -41,7 +49,7 @@ class Repo(object):
     from dvc.repo.add import add
     from dvc.repo.remove import remove
     from dvc.repo.ls import ls
-    from dvc.repo.lock import lock as lock_stage
+    from dvc.repo.freeze import freeze, unfreeze
     from dvc.repo.move import move
     from dvc.repo.run import run
     from dvc.repo.imp import imp
@@ -60,49 +68,64 @@ class Repo(object):
     from dvc.repo.get_url import get_url
     from dvc.repo.update import update
 
-    def __init__(self, root_dir=None):
-        from dvc.state import State
+    def __init__(self, root_dir=None, scm=None, rev=None):
+        from dvc.state import State, StateNoop
         from dvc.lock import make_lock
         from dvc.scm import SCM
         from dvc.cache import Cache
         from dvc.data_cloud import DataCloud
         from dvc.repo.metrics import Metrics
+        from dvc.repo.plots import Plots
+        from dvc.repo.params import Params
         from dvc.scm.tree import WorkingTree
-        from dvc.repo.tag import Tag
         from dvc.utils.fs import makedirs
+        from dvc.stage.cache import StageCache
 
-        root_dir = self.find_root(root_dir)
+        if scm:
+            # use GitTree instead of WorkingTree as default repo tree instance
+            tree = scm.get_tree(rev)
+            self.root_dir = self.find_root(root_dir, tree)
+            self.scm = scm
+            self.tree = tree
+            self.state = StateNoop()
+        else:
+            root_dir = self.find_root(root_dir)
+            self.root_dir = os.path.abspath(os.path.realpath(root_dir))
 
-        self.root_dir = os.path.abspath(os.path.realpath(root_dir))
         self.dvc_dir = os.path.join(self.root_dir, self.DVC_DIR)
-
         self.config = Config(self.dvc_dir)
 
-        no_scm = self.config["core"].get("no_scm", False)
-        self.scm = SCM(self.root_dir, no_scm=no_scm)
-
-        self.tree = WorkingTree(self.root_dir)
+        if not scm:
+            no_scm = self.config["core"].get("no_scm", False)
+            self.scm = SCM(self.root_dir, no_scm=no_scm)
+            self.tree = WorkingTree(self.root_dir)
 
         self.tmp_dir = os.path.join(self.dvc_dir, "tmp")
-        makedirs(self.tmp_dir, exist_ok=True)
+        self.index_dir = os.path.join(self.tmp_dir, "index")
+        makedirs(self.index_dir, exist_ok=True)
 
         hardlink_lock = self.config["core"].get("hardlink_lock", False)
         self.lock = make_lock(
-            os.path.join(self.dvc_dir, "lock"),
-            tmp_dir=os.path.join(self.dvc_dir, "tmp"),
+            os.path.join(self.tmp_dir, "lock"),
+            tmp_dir=self.tmp_dir,
             hardlink_lock=hardlink_lock,
             friendly=True,
         )
 
-        # NOTE: storing state and link_state in the repository itself to avoid
-        # any possible state corruption in 'shared cache dir' scenario.
-        self.state = State(self)
-
         self.cache = Cache(self)
         self.cloud = DataCloud(self)
 
+        if not scm:
+            # NOTE: storing state and link_state in the repository itself to
+            # avoid any possible state corruption in 'shared cache dir'
+            # scenario.
+            self.state = State(self.cache.local)
+
+        self.stage_cache = StageCache(self)
+
         self.metrics = Metrics(self)
-        self.tag = Tag(self)
+        self.plots = Plots(self)
+        self.params = Params(self)
 
         self._ignore()
 
@@ -112,20 +135,31 @@ class Repo(object):
 
     @tree.setter
     def tree(self, tree):
-        self._tree = tree if isinstance(tree, CleanTree) else CleanTree(tree)
+        if is_working_tree(tree) or tree.tree_root == self.root_dir:
+            root = None
+        else:
+            root = self.root_dir
+        self._tree = (
+            tree if isinstance(tree, CleanTree) else CleanTree(tree, root)
+        )
         # Our graph cache is no longer valid, as it was based on the previous
         # tree.
         self._reset()
 
     def __repr__(self):
-        return "{}: '{}'".format(self.__class__.__name__, self.root_dir)
+        return f"{self.__class__.__name__}: '{self.root_dir}'"
 
     @classmethod
-    def find_root(cls, root=None):
+    def find_root(cls, root=None, tree=None):
         root_dir = os.path.realpath(root or os.curdir)
 
+        if tree:
+            if tree.isdir(os.path.join(root_dir, cls.DVC_DIR)):
+                return root_dir
+            raise NotDvcRepoError(f"'{root}' does not contain DVC directory")
+
         if not os.path.isdir(root_dir):
-            raise NotDvcRepoError("directory '{}' does not exist".format(root))
+            raise NotDvcRepoError(f"directory '{root}' does not exist")
 
         while True:
             dvc_dir = os.path.join(root_dir, cls.DVC_DIR)
@@ -154,67 +188,145 @@ class Repo(object):
         return Repo(root_dir)
 
     def unprotect(self, target):
-        return self.cache.local.unprotect(PathInfo(target))
+        return self.cache.local.tree.unprotect(PathInfo(target))
 
     def _ignore(self):
-        from dvc.updater import Updater
-
-        updater = Updater(self.dvc_dir)
-
-        flist = (
-            [self.config.files["local"], updater.updater_file]
-            + [self.lock.lockfile, updater.lock.lockfile, self.tmp_dir]
-            + self.state.files
-        )
+        flist = [self.config.files["local"], self.tmp_dir]
 
         if path_isin(self.cache.local.cache_dir, self.root_dir):
             flist += [self.cache.local.cache_dir]
 
         self.scm.ignore_list(flist)
 
+    def get_stage(self, path=None, name=None):
+        if not path:
+            path = PIPELINE_FILE
+            logger.debug("Assuming '%s' to be a stage inside '%s'", name, path)
+
+        dvcfile = Dvcfile(self, path)
+        return dvcfile.stages[name]
+
+    def get_stages(self, path=None, name=None):
+        if not path:
+            path = PIPELINE_FILE
+            logger.debug("Assuming '%s' to be a stage inside '%s'", name, path)
+
+        if name:
+            return [self.get_stage(path, name)]
+
+        dvcfile = Dvcfile(self, path)
+        return list(dvcfile.stages.values())
+
     def check_modified_graph(self, new_stages):
         """Generate graph including the new stage to check for errors"""
-        self._collect_graph(self.stages + new_stages)
+        # Building graph might be costly for the ones with many DVC-files,
+        # so we provide this undocumented hack to skip it. See [1] for
+        # more details. The hack can be used as:
+        #
+        #     repo = Repo(...)
+        #     repo._skip_graph_checks = True
+        #     repo.add(...)
+        #
+        # A user should care about not duplicating outs and not adding cycles,
+        # otherwise DVC might have an undefined behaviour.
+        #
+        # [1] https://github.com/iterative/dvc/issues/2671
+        if not getattr(self, "_skip_graph_checks", False):
+            self._collect_graph(self.stages + new_stages)
 
-    def collect(self, target, with_deps=False, recursive=False, graph=None):
+    def _collect_inside(self, path, graph):
         import networkx as nx
-        from dvc.stage import Stage
 
+        stages = nx.dfs_postorder_nodes(graph)
+        return [stage for stage in stages if path_isin(stage.path, path)]
+
+    def collect(
+        self, target=None, with_deps=False, recursive=False, graph=None
+    ):
         if not target:
             return list(graph) if graph else self.stages
 
-        target = os.path.abspath(target)
-
         if recursive and os.path.isdir(target):
-            stages = nx.dfs_postorder_nodes(graph or self.graph)
-            return [stage for stage in stages if path_isin(stage.path, target)]
+            return self._collect_inside(
+                os.path.abspath(target), graph or self.graph
+            )
 
-        stage = Stage.load(self, target)
-
-        # Optimization: do not collect the graph for a specific target
+        path, name = parse_target(target)
+        stages = self.get_stages(path, name)
         if not with_deps:
-            return [stage]
+            return stages
+
+        res = set()
+        for stage in stages:
+            res.update(self._collect_pipeline(stage, graph=graph))
+        return res
+
+    def _collect_pipeline(self, stage, graph=None):
+        import networkx as nx
 
         pipeline = get_pipeline(get_pipelines(graph or self.graph), stage)
-        return list(nx.dfs_postorder_nodes(pipeline, stage))
+        return nx.dfs_postorder_nodes(pipeline, stage)
 
-    def collect_granular(self, target, *args, **kwargs):
-        from dvc.stage import Stage
+    def _collect_from_default_dvcfile(self, target):
+        dvcfile = Dvcfile(self, PIPELINE_FILE)
+        if dvcfile.exists():
+            return dvcfile.stages.get(target)
 
+    def collect_granular(
+        self, target=None, with_deps=False, recursive=False, graph=None
+    ):
+        """
+        Priority is in the order of following in case of ambiguity:
+            - .dvc file or .yaml file
+            - dir if recursive and directory exists
+            - stage_name
+            - output file
+        """
         if not target:
             return [(stage, None) for stage in self.stages]
 
-        # Optimization: do not collect the graph for a specific .dvc target
-        if Stage.is_valid_filename(target) and not kwargs.get("with_deps"):
-            return [(Stage.load(self, target), None)]
+        file, name = parse_target(target)
+        stages = []
 
-        try:
-            (out,) = self.find_outs_by_path(target, strict=False)
-            filter_info = PathInfo(os.path.abspath(target))
-            return [(out.stage, filter_info)]
-        except OutputNotFoundError:
-            stages = self.collect(target, *args, **kwargs)
-            return [(stage, None) for stage in stages]
+        # Optimization: do not collect the graph for a specific target
+        if not file:
+            # parsing is ambiguous when it does not have a colon
+            # or if it's not a dvcfile, as it can be a stage name
+            # in `dvc.yaml` or, an output in a stage.
+            logger.debug(
+                "Checking if stage '%s' is in '%s'", target, PIPELINE_FILE
+            )
+            if not (recursive and os.path.isdir(target)):
+                stage = self._collect_from_default_dvcfile(target)
+                if stage:
+                    stages = (
+                        self._collect_pipeline(stage) if with_deps else [stage]
+                    )
+        elif not with_deps and is_valid_filename(file):
+            stages = self.get_stages(file, name)
+
+        if not stages:
+            if not (recursive and os.path.isdir(target)):
+                try:
+                    (out,) = self.find_outs_by_path(target, strict=False)
+                    filter_info = PathInfo(os.path.abspath(target))
+                    return [(out.stage, filter_info)]
+                except OutputNotFoundError:
+                    pass
+
+            try:
+                stages = self.collect(target, with_deps, recursive, graph)
+            except StageFileDoesNotExistError as exc:
+                # collect() might try to use `target` as a stage name
+                # and throw error that dvc.yaml does not exist, whereas it
+                # should say that both stage name and file does not exist.
+                if file and is_valid_filename(file):
+                    raise
+                raise NoOutputOrStageError(target, exc.file) from exc
+            except StageNotFound as exc:
+                raise NoOutputOrStageError(target, exc.file) from exc
+
+        return [(stage, None) for stage in stages]
 
     def used_cache(
         self,
@@ -227,6 +339,7 @@ class Repo(object):
         force=False,
         jobs=None,
         recursive=False,
+        used_run_cache=None,
     ):
         """Get the stages related to the given target and collect
         the `info` of its outputs.
@@ -238,8 +351,9 @@ class Repo(object):
         `all_branches`/`all_tags`/`all_commits` to expand the scope.
 
         Returns:
-            A dictionary with Schemes (representing output's location) as keys,
-            and a list with the outputs' `dumpd` as values.
+            A dictionary with Schemes (representing output's location) mapped
+            to items containing the output's `dumpd` names and the output's
+            children (if the given output is a directory).
         """
         from dvc.cache import NamedCache
 
@@ -259,7 +373,7 @@ class Repo(object):
                 for target in targets
             )
 
-            suffix = "({})".format(branch) if branch else ""
+            suffix = f"({branch})" if branch else ""
             for stage, filter_info in pairs:
                 used_cache = stage.get_used_cache(
                     remote=remote,
@@ -269,9 +383,15 @@ class Repo(object):
                 )
                 cache.update(used_cache, suffix=suffix)
 
+        if used_run_cache:
+            used_cache = self.stage_cache.get_used_cache(
+                used_run_cache, remote=remote, force=force, jobs=jobs,
+            )
+            cache.update(used_cache)
+
         return cache
 
-    def _collect_graph(self, stages=None):
+    def _collect_graph(self, stages):
         """Generate a graph by using the given stages on the given directory
 
         The nodes of the graph are the stage's path relative to the root.
@@ -320,10 +440,9 @@ class Repo(object):
 
         G = nx.DiGraph()
         stages = stages or self.stages
-        stages = [stage for stage in stages if stage]
         outs = Trie()  # Use trie to efficiently find overlapping outs and deps
 
-        for stage in stages:
+        for stage in filter(bool, stages):  # bug? not using it later
             for out in stage.outs:
                 out_key = out.path_info.parts
 
@@ -346,9 +465,9 @@ class Repo(object):
                         "rerun command with non overlapping outs paths."
                     ).format(
                         str(parent),
-                        parent.stage.relpath,
+                        parent.stage.addressing,
                         str(overlapping),
-                        overlapping.stage.relpath,
+                        overlapping.stage.addressing,
                     )
                     raise OverlappingOutputPathsError(parent, overlapping, msg)
 
@@ -367,19 +486,18 @@ class Repo(object):
                     continue
 
                 dep_key = dep.path_info.parts
-                overlapping = list(n.value for n in outs.prefixes(dep_key))
+                overlapping = [n.value for n in outs.prefixes(dep_key)]
                 if outs.has_subtrie(dep_key):
                     overlapping.extend(outs.values(prefix=dep_key))
 
                 G.add_edges_from((stage, out.stage) for out in overlapping)
-
         check_acyclic(G)
 
         return G
 
     @cached_property
     def graph(self):
-        return self._collect_graph()
+        return self._collect_graph(self.stages)
 
     @cached_property
     def pipelines(self):
@@ -396,25 +514,29 @@ class Repo(object):
         NOTE: For large repos, this could be an expensive
               operation. Consider using some memoization.
         """
-        from dvc.stage import Stage
+        return self._collect_stages()
 
+    @cached_property
+    def plot_templates(self):
+        from .plots.template import PlotTemplates
+
+        return PlotTemplates(self.dvc_dir)
+
+    def _collect_stages(self):
         stages = []
         outs = set()
 
         for root, dirs, files in self.tree.walk(self.root_dir):
-            for fname in files:
-                path = os.path.join(root, fname)
-                if not Stage.is_valid_filename(path):
-                    continue
-                stage = Stage.load(self, path)
-                stages.append(stage)
-
-                for out in stage.outs:
-                    if out.scheme == "local":
-                        outs.add(out.fspath)
-
+            for file_name in filter(is_valid_filename, files):
+                new_stages = self.get_stages(os.path.join(root, file_name))
+                stages.extend(new_stages)
+                outs.update(
+                    out.fspath
+                    for stage in new_stages
+                    for out in stage.outs
+                    if out.scheme == "local"
+                )
             dirs[:] = [d for d in dirs if os.path.join(root, d) not in outs]
-
         return stages
 
     def find_outs_by_path(self, path, outs=None, recursive=False, strict=True):
@@ -452,49 +574,22 @@ class Repo(object):
     @contextmanager
     def open_by_relpath(self, path, remote=None, mode="r", encoding=None):
         """Opens a specified resource as a file descriptor"""
-        cause = None
+
+        tree = RepoTree(self, stream=True)
+        path = os.path.join(self.root_dir, path)
         try:
-            out = self.find_out_by_relpath(path)
-        except OutputNotFoundError as exc:
-            out = None
-            cause = exc
-
-        if out and out.use_cache:
-            try:
-                with self._open_cached(out, remote, mode, encoding) as fd:
-                    yield fd
-                return
-            except FileNotFoundError as exc:
-                raise FileMissingError(path) from exc
-
-        abs_path = os.path.join(self.root_dir, path)
-        if os.path.exists(abs_path):
-            with open(abs_path, mode=mode, encoding=encoding) as fd:
-                yield fd
-            return
-
-        raise FileMissingError(path) from cause
-
-    def _open_cached(self, out, remote=None, mode="r", encoding=None):
-        if out.isdir():
-            raise IsADirectoryError("Can't open a dir")
-
-        cache_file = self.cache.local.checksum_to_path_info(out.checksum)
-        cache_file = fspath_py35(cache_file)
-
-        if os.path.exists(cache_file):
-            return open(cache_file, mode=mode, encoding=encoding)
-
-        try:
-            remote_obj = self.cloud.get_remote(remote)
-            remote_info = remote_obj.checksum_to_path_info(out.checksum)
-            return remote_obj.open(remote_info, mode=mode, encoding=encoding)
-        except RemoteActionNotImplemented:
             with self.state:
-                cache_info = out.get_used_cache(remote=remote)
-                self.cloud.pull(cache_info, remote=remote)
-
-            return open(cache_file, mode=mode, encoding=encoding)
+                with tree.open(
+                    os.path.join(self.root_dir, path),
+                    mode=mode,
+                    encoding=encoding,
+                    remote=remote,
+                ) as fobj:
+                    yield fobj
+        except FileNotFoundError as exc:
+            raise FileMissingError(path) from exc
+        except IsADirectoryError as exc:
+            raise DvcIsADirectoryError(f"'{path}' is a directory") from exc
 
     def close(self):
         self.scm.close()
